@@ -7,18 +7,22 @@
 
 from rdfhelpers.rdfhelpers import graphFrom, expandQName, URI, getvalue, setvalue
 from rdfhelpers.rdfhelpers import isContainerItemPredicate, makeContainerItemPredicate, diff
+from rdfhelpers.rdfhelpers import cbd, cbdAnalysis
 from rdfhelpers.rdfhelpers import getContainerStatements, getContainerItems, setContainerItems
 from rdfhelpers.rdfhelpers import SPARQLRepository, JournaledGraph, FocusedGraph
 from rdfhelpers.templated import identity, mapDict, Templated, TemplatedQueryMixin
 from rdfhelpers.constructor import Constructor
 from rdfhelpers.labels import LabelCache, SKOSLabelCache
+from rdfhelpers.gsp import GraphStoreClient
 
 __all__ = [
     'graphFrom', 'expandQName', 'URI', 'getvalue', 'setvalue',
     'isContainerItemPredicate', 'makeContainerItemPredicate', 'diff',
+    'cbd', 'cbdAnalysis',
     'getContainerStatements', 'getContainerItems', 'setContainerItems',
     'SPARQLRepository', 'JournaledGraph', 'FocusedGraph',
     'identity', 'mapDict', 'Templated', 'TemplatedQueryMixin',
     'Constructor',
-    'LabelCache', 'SKOSLabelCache'
+    'LabelCache', 'SKOSLabelCache',
+    'GraphStoreClient'
 ]
