@@ -1,5 +1,13 @@
-from rdflib import URIRef, BNode, Graph, RDF
-from rdflib.plugins.stores.sparqlstore import SPARQLUpdateStore
+# Copyright (c) 2022, Ora Lassila & So Many Aircraft
+# All rights reserved.
+#
+# See LICENSE for licensing information
+#
+# This module implements some useful functionality for programming with RDFLib.
+#
+
+from rdflib import URIRef, BNode, Variable, Graph, RDF, ConjunctiveGraph
+from rdflib.plugins.stores.sparqlstore import SPARQLStore, SPARQLUpdateStore
 from rdfhelpers.templated import Templated, TemplatedQueryMixin
 import re
 
@@ -28,6 +36,69 @@ def expandQName(prefix, local_name, ns_mgr):
 def URI(u):
     return u if isinstance(u, URIRef) or u is None else URIRef(u)
 
+CBD_QUERY = '''
+    SELECT ?s1 ?p1 ?o1 {
+        GRAPH $context {
+            {
+                SELECT ?s1 ?p1 ?o1 {
+                    BIND($uri AS ?s1)
+                    ?s1 ?p1 ?o1
+                }
+            } UNION {
+                SELECT ?s1 ?p1 ?o1 {
+                    BIND($uri AS ?s2)
+                    ?s2 ?p2 ?s1 . FILTER(ISBLANK(?s1))
+                    ?s1 ?p1 ?o1
+                }
+            } UNION {
+                SELECT ?s1 ?p1 ?o1 {
+                    BIND($uri AS ?s3)
+                    ?s3 ?p3 ?s2 . FILTER(ISBLANK(?s2))
+                    ?s2 ?p2 ?s1 . FILTER(ISBLANK(?s1))
+                    ?s1 ?p1 ?o1
+                }
+            }
+        }
+    }
+'''
+
+def cbd(source, target, resource, context):
+    # Does not support reified statements (yet)
+    if isinstance(source, SPARQLStore):
+        if context is None:
+            raise NotImplementedError("CBD via SPARQL and no context")
+        return graphFrom(Templated.query(source, CBD_QUERY, context=URIRef(context), uri=resource),
+                         add_to=target or Graph())
+    else:
+        if target is None:
+            target = source.__class__()
+        subjects = list([resource])
+        if isinstance(source, ConjunctiveGraph):
+            def get_triples(s): return source.quads((s, None, None, context))
+            if context is None and not isinstance(target, ConjunctiveGraph):
+                raise ValueError("Target must be context-aware: %s".format(target))
+        else:
+            def get_triples(s): return source.triples((s, None, None))
+        while subjects:
+            subject = subjects.pop()
+            for triple in get_triples(subject):
+                target.add(triple)
+                leaf = triple[2]
+                if isinstance(leaf, BNode):
+                    subjects.append(leaf)
+        return target
+
+CBD_ANALYSIS_QUERY = '''
+    SELECT ?s ?p ?o {
+        ?s ?p ?o .
+        FILTER isBlank(?o)
+        FILTER NOT EXISTS { ?o ?pp ?oo }
+    }
+'''
+
+def cbdAnalysis(self, candidate_cbd):
+    return candidate_cbd.query(self.CBD_ANALYSIS_QUERY)
+
 # Helpful graph accessors
 
 def getvalue(graph, node, predicate):
@@ -104,6 +175,10 @@ def setContainerItems(graph, node, predicate, values, newtype=RDF.Seq):
 #
 # Instances of FocusedGraph have a specified "focus node".
 
+def ntriples(graph, destination=None):
+    result = graph.serialize(destination=destination, format="nt", encoding="utf-8")
+    return result.decode("utf-8") if destination is None else None
+
 class FocusedGraph(Graph):
     def __init__(self, focus=None, source=None, focus_class=None, **kwargs):
         super().__init__(**kwargs)
@@ -128,6 +203,33 @@ class FocusedGraph(Graph):
     def setvalue(self, predicate, value):
         setvalue(self, self._focus, predicate, value)
 
+# CBD GRAPH
+
+class CBDGraph(FocusedGraph):
+    def __init__(self, focus, data, context=None, **kwargs):
+        super().__init__(focus=focus, **kwargs)
+        self.data = data
+        self.context = context
+        cbd(data, self, focus, context)
+
+    FLUSHUPDATE = '''
+        DELETE DATA { $remove_stuff };
+        INSERT DATA { $add_stuff }
+    '''
+    FLUSHUPDATE_WITH_CONTEXT = '''
+        DELETE DATA { GRAPH $context { $remove_stuff } };
+        INSERT DATA { GRAPH $context { $add_stuff } }
+    '''
+
+    def flush(self):
+        context = self.context
+        additions, retractions = diff(self, cbd(self, None, self.focus, context))
+        if len(additions) > 0 or len(retractions) > 0:
+            Templated.update(self.data,
+                             self.FLUSHUPDATE if context is None else self.FLUSHUPDATE_WITH_CONTEXT,
+                             context=context,
+                             add_stuff=ntriples(additions), remove_stuff=ntriples(retractions))
+
 # SPARQL REPOSITORY
 
 class SPARQLRepository(TemplatedQueryMixin, SPARQLUpdateStore):
@@ -146,19 +248,19 @@ class SPARQLRepository(TemplatedQueryMixin, SPARQLUpdateStore):
 # capable of "flushing" these modifications to a backing store with a SPARQL endpoint.
 
 class JournaledGraph(Graph):
-    def __init__(self, store : SPARQLUpdateStore=None):
-        super().__init__()
+    def __init__(self, store: SPARQLUpdateStore = None, **kwargs):
+        super().__init__(**kwargs)
         self._journal = list()
         self._store = store
 
     def add(self, triple):
         super().add(triple)
-        self._journal.append((True, triple))
+        self._journal.append(("add", triple))
         return self
 
     def remove(self, triple):
         super().remove(triple)
-        self._journal.append((False, triple))
+        self._journal.append(("remove_all" if triple[2] is None else "remove", triple))
         return self
 
     def addN(self, quads):
@@ -171,25 +273,32 @@ class JournaledGraph(Graph):
             if context is None:
                 add_update = "INSERT DATA { $stuff }"
                 remove_update = "DELETE DATA { $stuff }"
+                remove_all_update = "DELETE WHERE { $stuff }"
             else:
                 add_update = "INSERT DATA { GRAPH $context { $stuff } }"
                 remove_update = "DELETE DATA { GRAPH $context { $stuff } }"
+                remove_all_update = "DELETE WHERE { GRAPH $context { $stuff } }"
             mods = list()
             case = self._journal[0][0]
             current_mods = Graph()
-            for added, triple in self._journal:
-                if added != case:
+            for new_case, triple in self._journal:
+                if new_case != case:
                     mods.append((case, current_mods))
-                    case = added
+                    case = new_case
                     current_mods = Graph()
                 current_mods.add(triple)
             mods.append((case, current_mods))
-            self._store.update(";".join([Templated.convert(add_update if added else remove_update,
-                                                           {"stuff": self.ntriples(triples),
-                                                            "context": context})
-                                         for added, triples in mods]))  # no replacement necessary
+            updates = list()
+            for case, triples in mods:
+                if case == "add" or case == "remove":
+                    updates.append(Templated.convert(add_update if case == "add" else remove_update,
+                                                     {"stuff": ntriples(triples),
+                                                      "context": context}))
+                else:
+                    for triple in triples:
+                        g = graphFrom(list((triple[0], triple[1], Variable("x"))))
+                        updates.append(Templated.convert(remove_all_update,
+                                                         {"stuff": ntriples(g),
+                                                          "context": context}))
+            self._store.update(";".join(updates))
             self._journal = list()
-
-    @staticmethod
-    def ntriples(graph):
-        return graph.serialize(format="ntriples", encoding="ascii").decode()
